local rc = require 'rclib-ortho'

local speed = 120 -- pix/s
local alphaThreshold = 0.4 -- Alpha value under which pixels are considered fully transparent

local tree, hero
local px, py = 0, 0
local lg = love.graphics

local shaderTemplate = [[
  vec4 effect(vec4 colour, Image img, vec2 imgpos, vec2 srcpos)
  {
    vec4 result = Texel(img, imgpos) * colour;
    if (result.a < %g)
      discard;
    return result;
  }
]]

local function newSprite(filename)
  local img = lg.newImage(filename)
  local sprite = rc(img)
  local w, h = img:getDimensions()
  sprite:rect3d(0, 0, h/3000,
                0, h, 0,
                w, 0, h/3000,
                0, 0,  w, h,  w, h)
  return sprite
end

function love.load()
  tree = newSprite('tree.png')
  hero = newSprite('hero.png')
  lg.setDepthMode("lequal", true)
  lg.setShader(lg.newShader(shaderTemplate:format(alphaThreshold)))
end

function love.update(dt)
  if love.keyboard.isDown("left") then
    px = px - speed * dt
  end
  if love.keyboard.isDown("right") or pressed2 then
    px = px + speed * dt
  end
  if love.keyboard.isDown("up") or pressed3 then
    py = py - speed * dt
  end
  if love.keyboard.isDown("down") or pressed4 then
    py = py + speed * dt
  end
end

function love.draw()
  hero:draw(px, py)
  tree:draw(80, 10)
  tree:draw(530, 50)
  tree:draw(150, 250)
  tree:draw(620, 380)
end


function love.keypressed(k)
  if k == "escape" then return love.event.quit() end
end
