-- Planar mesh drawing lib
local lg = love.graphics

-- Maximum number of vertices - preferably a multiple of 4
local vmax = 4

local atypes = {
  {"VertexPosition", "float", 3},
  {"VertexTexCoord", "float", 2},
}

local vmap = {}

local function rect3d(rc, x1, y1, z1, x2, y2, z2, x3, y3, z3, u1, v1, u2, v2, w, h)
  local vnum = rc.vnum
  if vnum + 4 > vmax then
    error("rect3d maximum capacity exceeded")
  end
  vnum = vnum + 4
  rc.vnum = vnum
  local u, v = u1 / w, v1 / h
  rc.mesh:setVertex(vnum-3, x1, y1, z1, u, v)
  local vv = v
  v = v2 / h
  rc.mesh:setVertex(vnum-2, x2, y2, z2, u, v)
  u = u2 / w
  rc.mesh:setVertex(vnum-1, x3, y3, z3, u, vv)
  rc.mesh:setVertex(vnum, x2 + x3 - x1, y2 + y3 - y1, z2 + z3 - z1, u, v)

  vmap[(vnum-4)*1.5 + 1] = vnum - 3
  vmap[(vnum-4)*1.5 + 2] = vnum - 2
  vmap[(vnum-4)*1.5 + 3] = vnum - 1
  vmap[(vnum-4)*1.5 + 4] = vnum - 1
  vmap[(vnum-4)*1.5 + 5] = vnum - 2
  vmap[(vnum-4)*1.5 + 6] = vnum - 0
end

local function draw(rc, x, y)
  if rc.vnum ~= rc.oldvnum then
    -- added rectangles
    rc.mesh:setDrawRange(1, #vmap)
    rc.mesh:setVertexMap(vmap)
    rc.oldvnum = vnum
  end
  lg.draw(rc.mesh, x or 0, y or 0)
end

local function init(tex)
  local rc = {
    rect3d = rect3d;
    draw = draw;
    vnum = 0;
    oldvnum = 0;
  }
  rc.mesh = lg.newMesh(atypes, vmax, "triangles", "dynamic")
  rc.mesh:setTexture(tex)

  oldvnum, vnum = -1, 0
  -- clear vmap
  for i = #vmap, 1, -1 do vmap[i] = nil end
  return rc
end

return init
